/*
 * file UcmHelper.java
 * 
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.UcmHelper
 *
 * (C) Copyright IBM Corporation 2008, 2010.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.cmapi.samples.cc;

import javax.wvcm.WvcmException;
import javax.wvcm.PropertyRequestItem.PropertyRequest;
import javax.wvcm.WvcmException.ReasonCode;

import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpLocation.Namespace;
import com.ibm.rational.wvcm.stp.StpProvider.Domain;
import com.ibm.rational.wvcm.stp.cc.CcProject;
import com.ibm.rational.wvcm.stp.cc.CcProjectFolder;
import com.ibm.rational.wvcm.stp.cc.CcProvider;
import com.ibm.rational.wvcm.stp.cc.CcResource;
import com.ibm.rational.wvcm.stp.cc.CcStream;
import com.ibm.rational.wvcm.stp.cc.CcVob;

public class UcmHelper {

    private final CcProvider m_provider;
    private CcStream m_devStream;
    private CcStream m_intStream;
    private CcProject m_project;
    private CcProjectFolder m_rootFolder;
    private final String m_pvobTag;
    private final String m_loginName;
    private CcDemoBase m_demo;
    private CcVob m_pvob;

    public UcmHelper(CcDemoBase demo) throws Exception {
        m_demo = demo;
        m_provider = demo.getClearCaseProvider();
        m_loginName = demo.props().getLoginName();
        m_pvobTag = demo.props().getDemoUcmProjectVob();

        // Verify project VOB exists.
        getUcmProjectVob();
    }

    public CcStream getUcmDevelopmentStream() throws Exception {
        if (m_devStream == null) {

            // Construct location object for development stream.
            StpLocation streamLoc = m_provider.userFriendlySelector(
                    Domain.CLEAR_CASE,
                    Namespace.STREAM,
                    m_loginName + ".dev.stream",
                    m_pvobTag);

            // Use provider's factory method to construct CC stream proxy.
            // Note: This call does not contact the server. We don't know
            // at this point whether the stream exists or not.
            m_devStream = m_provider.ccStream(streamLoc);

            if ( ! resourceExists(m_devStream)) {
                // The development stream doesn't exist yet.  Create it.
                CcStream intStream = getUcmIntegrationStream();
                m_devStream.setParentStream(intStream);
                trace("Creating UCM demo development stream: " + m_devStream);
                m_devStream = m_devStream.doCreateCcStream(null);
            }
        }
        return m_devStream;
    }

    public CcStream getUcmIntegrationStream() throws Exception {
        if (m_intStream == null) {

            // Construct location object for integration stream.
            StpLocation streamLoc = m_provider.userFriendlySelector(
                    Domain.CLEAR_CASE,
                    Namespace.STREAM,
                    m_loginName + ".int.stream",
                    m_pvobTag);

            // Use provider's factory method to construct CC stream proxy.
            // Note: This call does not contact the server. We don't know
            // at this point whether the stream exists or not.
            m_intStream = m_provider.ccStream(streamLoc);

            if ( ! resourceExists(m_intStream)) {
                // The integration stream doesn't exist yet.  Create it.
                CcProject project = getUcmProject();
                m_intStream.setParentProject(project);
                m_intStream.setIsIntegrationStream(true);
                trace("Creating UCM demo integration stream: " + m_intStream);
                m_intStream = m_intStream.doCreateCcStream(null);
            }
        }
        return m_intStream;
    }

    public CcProject getUcmProject() throws Exception {
        if (m_project == null) {

            // Construct location object for UCM project.
            StpLocation projectLoc = m_provider.userFriendlySelector(
                    Domain.CLEAR_CASE,
                    Namespace.PROJECT,
                    m_loginName + ".project",
                    m_pvobTag);

            // Use provider's factory method to construct project proxy.
            // Note: This call does not contact the server. We don't know
            // at this point whether the project exists or not.
            m_project = m_provider.ccProject(projectLoc);

            if ( ! resourceExists(m_project)) {
                // The project doesn't exist yet.  Create it.
                CcProjectFolder folder = getUcmRootFolder();
                m_project.setProjectFolder(folder);
                trace("Creating UCM demo project: " + m_project);
                m_project = m_project.doCreateCcProject(null);
            }
        }
        return m_project;
    }

    public CcProjectFolder getUcmRootFolder() throws Exception {
        if (m_rootFolder == null) {

            // Construct location object for the root project folder.
            StpLocation folderLoc = m_provider.userFriendlySelector(
                    Domain.CLEAR_CASE,
                    Namespace.FOLDER,
                    "RootFolder",
                    m_pvobTag);

            m_rootFolder = m_provider.ccProjectFolder(folderLoc);

            if ( ! resourceExists(m_rootFolder)) {
                throw new IllegalStateException("Missing root project folder: " + m_rootFolder);
            }
        }
        return m_rootFolder;
    }

    public CcVob getUcmProjectVob() throws Exception {
        if (m_pvob == null) {

            // Construct location object for the root project folder.
            StpLocation pvobLoc = m_provider.userFriendlySelector(
                    Domain.CLEAR_CASE,
                    Namespace.VOB,
                    null,
                    m_pvobTag);

            m_pvob = m_provider.ccVob(pvobLoc);
            PropertyRequest wantedProps = new PropertyRequest(CcVob.IS_PROJECT_VOB);
            m_pvob = (CcVob) m_pvob.doReadProperties(wantedProps);
            if ( ! m_pvob.getIsProjectVob()) {
                throw new IllegalStateException("Not a UCM project VOB: " + m_pvob);
            }
        }
        return m_pvob;
    }

    private boolean resourceExists(CcResource res) throws Exception {
        try {
            res = (CcResource) res.doReadProperties(null);
            return true;
        } catch (WvcmException ex) {
            if (ex.getReasonCode() == ReasonCode.NOT_FOUND) {
                return false;
            } else {
                throw ex;
            }
        }
    }

    private void trace(String msg) {
        m_demo.trace(msg);
    }
}
